/*   FILE: common.c -- 
 * AUTHOR: W. Michael Petullo <new@flyn.org>
 *   DATE: 01 MAY 1 999
 *
 * Copyright (c) 1999 W. Michael Petullo <new@flyn.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <limits.h>
#include <common.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <config.h>
#include <sys/stat.h>
#include <template.h>
#ifdef HAVE_PWDB_PWDB_PUBLIC_H
#include <pwdb/pwdb_public.h>
#else
#include <pwd.h>
#endif				/* HAVE_PWDB_PWDB_PUBLIC_H */

#ifdef HAVE_PWDB_PWDB_PUBLIC_H
/* ============================ _get_pwdb_entry () ========================= */
const struct pwdb_entry *get_pwdb_entry(const int uid, const char *field)
{
    const struct pwdb *p = NULL;
    const struct pwdb_entry *e = NULL;
    pwdb_locate("user", PWDB_DEFAULT, PWDB_NAME_UNKNOWN, uid, &p);
    pwdb_get_entry(p, field, &e);
    return e;
}
#endif				/* HAVE_PWDB_PWDB_PUBLIC_H */

/* ============================ homedir () ================================== */
char *homedir(char *homedir)
{
    int uid = getuid();
#ifdef HAVE_PWDB_PWDB_PUBLIC_H
    const struct pwdb_entry *e = get_pwdb_entry(uid, "dir");
    strncpy(homedir, e->value ? e->value : "", PATH_MAX);
    return e->value ? homedir : NULL;
#else
    struct passwd *p = getpwuid(uid);
    strncpy(homedir, p->pw_dir ? p->pw_dir : "", PATH_MAX);
    return p->pw_gecos ? homedir : NULL;
#endif				/* HAVE_PWDB */
}

/* ============================ today () ==================================== */
char *today(char *date)
/* Make sure date is at least large enough to hold "10 September 1999". */
{
    time_t sec_since_1970;
    struct tm *curr_time;
    time(&sec_since_1970);
    curr_time = localtime(&sec_since_1970);
    strftime(date, 18, "%d %B %Y", curr_time);
    return date;
}

/* ============================ print_dir () ================================ */
void print_dir(DIR * dp)
{
    struct dirent *ent;
    while ((ent = readdir(dp))) {
	if (!(strcmp(ent->d_name, ".") && strcmp(ent->d_name, "..")))
	    continue;
	printf("  %s\n", ent->d_name);
    }
}

/* ============================= initialize_fillers () ====================== */
void initialize_fillers(fmt_ptrn_t *x)
{
    int i;
    char buf[BUFSIZ + 1];
    /* Read environ. */
    for (i = 0; environ[i] != 0x00; i++) {
        /* No longer uses strtok; non-destructive. */
	char *delim = strchr (environ[i], '=');  
	size_t len = delim - environ[i];
	char *key = (char *) malloc (sizeof (char) * (len + 1)), *val;
	strncpy (key, environ[i], len);
	key[len] = 0x00;
	val = strdup(delim + 1);
	if (val && key)
	    fmt_ptrn_update_mapping(x, key, val);
    }
    fmt_ptrn_update_mapping(x, strdup("DATE"), strdup(today(buf)));
    fmt_ptrn_update_mapping(x, strdup("FULLNAME"), strdup(fullname(buf)));
    fmt_ptrn_update_mapping(x, strdup("EMPTY_STR"), strdup(""));
}

/* ============================ _template_write () ========================== */
static int _template_write(fmt_ptrn_t *template, FILE *output_file)
{
    char b[BUFSIZ];
    while (fmt_ptrn_gets(b, BUFSIZ, template))
	fprintf(output_file, "%s", b);
    return 1;
}

/* ============================ mk_parent_dirs () ========================== */ 
static int mk_parent_dirs(const char *path)
{
    char path_copy[PATH_MAX + 1], *delim, *ptr;
    struct stat stat_buf;
    mode_t mode = 0777;
    ptr = strcpy (path_copy, path);
    while ((delim = strchr (ptr, '/'))) {
        if (! strcspn (ptr, "/")) {
	    // Found leading '/.'
	    ptr++;
	    continue;
	}
        *delim = 0x00; 
        ptr = delim + 1;
        if (stat (path_copy, &stat_buf) == -1)
            if (mkdir (path_copy, mode) < 0)
                return 0;
        *delim = '/';
    }
    return 1;
}

/* ============================ write_it () ================================ */ 
int write_it (const char *filepath, const int force, const char *template_path)
{
	fmt_ptrn_t template;
	struct stat stat_buf;
	FILE *output_file;
	if (!force && stat(filepath, &stat_buf) != -1) {
	    fprintf(stderr, "%s exists\n", filepath);
	    return 0;
	}
	if (!mk_parent_dirs(filepath)) {
            fprintf(stderr, "could not create parent dirs for %s\n", filepath);
            return 0;
        }
	if (!(output_file = fopen(filepath, "w"))) {
	    fprintf(stderr, "could not open %s\n", filepath);
	    return 0;
	}
	if (!fmt_ptrn_open(template_path, &template)) {
	    fprintf(stderr, "could not open template %s\n", template_path);
	    return 0;
	}
	fmt_ptrn_update_mapping(&template, strdup("FILE"),
				strdup(basename(filepath)));
	initialize_fillers(&template);
	if (!_template_write(&template, output_file)) {
	    template_perror("error writing template");
	    exit(1);
	}
	while (fmt_ptrn_parse_err(&template))
	    fmt_ptrn_parse_perror(&template, NULL);
        fmt_ptrn_close(&template);
        fclose(output_file);
	return 1;
}	    
