/*   FILE: template.c --
 * AUTHOR: W. Michael Petullo <new@flyn.org>
 *   DATE: 01 MAY 1999
 *
 * Copyright (c) 1999 W. Michael Petullo <new@flyn.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <limits.h>
#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include <sys/stat.h>
#include <common.h>
#include <unistd.h>
#include <config.h>
#ifdef HAVE_PWDB_PWDB_PUBLIC_H
#include <pwdb/pwdb_public.h>
#else
#include <pwd.h>
#endif				/* HAVE_PWDB_PWDB_PUBLIC_H */

/* Globals. */
static char _template_errmsg[BUFSIZ + 1];
static char _template_local_dir[PATH_MAX + 1];
static char _template_global_dir[PATH_MAX + 1];

/* ============================ fullname () ================================= */
char *fullname(char *fullname)
{
    char *comma;
    int uid = getuid();
#ifdef HAVE_PWDB_PWDB_PUBLIC_H
    const struct pwdb_entry *e = get_pwdb_entry(uid, "gecos");
    if (! (e && e->value))
	sprintf(_template_errmsg, "could not get pwdb entry");
    strncpy(fullname, e->value ? e->value : "", BUFSIZ);
#else
    struct passwd *p = getpwuid(uid);
    if (! (p && p->pw_gecos))
	sprintf(_template_errmsg, "could not get pwdb entry");
    strncpy(fullname, p->pw_gecos ? p->pw_gecos : "", BUFSIZ);
#endif				/* HAVE_PWDB_PWDB_PUBLIC_H */
    comma = strchr(fullname, ',');
    if (comma)
	*comma = 0x00;
#ifdef HAVE_PWDB_PWDB_PUBLIC_H
    return e->value ? fullname : NULL;
#else
    return p->pw_gecos ? fullname : NULL;
#endif				/* HAVE_PWDB_PWDB_PUBLIC_H */
}

/* ============================ template_init () ============================ */
void template_init(void)
{
    strcpy(_template_errmsg, "no error");
#ifdef HAVE_PWDB_PWDB_PUBLIC_H
    if (pwdb_start() != PWDB_SUCCESS)
	sprintf(_template_errmsg, "could not start pwdb lib");
#endif
}

/* ============================ template_destroy () ========================= */
void template_destroy(void)
{
#ifdef HAVE_PWDB_PWDB_PUBLIC_H
    pwdb_end();
#endif
}

/* ============================ _set_file_type () =========================== */
static int _set_file_type(char *type, const char *filename)
{
    char *dot = strrchr(filename, '.');
    if (dot)
	strcpy(type, dot == filename ? dot : ++dot);
    return dot ? 1 : 0;
}

/* ============================ template_set_local_dir () =================== */
int template_set_local_dir(const char *d)
{
    char dir[PATH_MAX + 1];
    if (!homedir(dir)) {
	sprintf(_template_errmsg, "could not get homedir");
	return 0;
    }
    strcpy(_template_local_dir, dir);
    strcat(_template_local_dir, "/");
    strcat(_template_local_dir, d);
    return 1;
}

/* ============================ template_set_global_dir () ================== */
int template_set_global_dir(const char *path)
{
    strncpy(_template_global_dir, path, PATH_MAX);
    return 1;
}

/* ============================ _build_template_dir () ====================== */
static void _build_template_dir(char *path, const char *dir,
				const char *type)
{
    strcpy(path, dir);
    if (type) {
	strcat(path, "/");
	strcat(path, type);
    }
    strcat(path, "/");
}

/* ============================ _at_path () ================================= */
static int _at_path(char *template_path)
/* Make sure template_path is big enough to have .gz added to the end! */
{
    struct stat stat_buf;
    if (stat(template_path, &stat_buf) != -1)
	return 1;
    else {
	strcat(template_path, ".gz");
	if (stat(template_path, &stat_buf) != -1)
	    return 1;
    }
    return 0;
}

/* ============================ template_find () ============================ */
int template_find(char *template_path, const char *filename,
			  const char *template_name, const int use_global)
{
    char type[PATH_MAX + 1];
    if (!(_set_file_type(type, filename)))
	strcpy(type, filename);
    if (!use_global) {
	/* Try $HOME/.new/templates first. */
	_build_template_dir(template_path, _template_local_dir, type);
	strcat(template_path, template_name);
    }
    if (((!use_global) && !_at_path(template_path))
	|| use_global) {
	_build_template_dir(template_path, _template_global_dir, type);
	strcat(template_path, template_name);
    }
    if (!_at_path(template_path)) {
	sprintf(_template_errmsg, "no template for type %s", type);
	return 0;
    }
    return 1;
}

/* ============================ template_list () ============================ */
int template_list(const char *filename)
{
    char template_path[PATH_MAX + 1], type[PATH_MAX + 1];
    DIR *dp;
    if (!(_set_file_type(type, filename)))
	strcpy(type, filename);
    printf("Personal templates for file type .%s:\n", type);
    /* Try $HOME/.new/templates first. */
    _build_template_dir(template_path, _template_local_dir, type);
    if (!(dp = opendir(template_path)))
	printf("  <none>\n");
    else
	print_dir(dp);
    closedir(dp);
    printf("\nGlobal templates for file type .%s:\n", type);
    /* Try global location. */
    _build_template_dir(template_path, _template_global_dir, type);
    if (!(dp = opendir(template_path)))
	printf("  <none>\n");
    else
	print_dir(dp);
    closedir(dp);
    return 1;
}

/* ============================ template_perror () ========================== */
void template_perror(const char *msg)
{
    fprintf(stderr, "%s: %s\n", msg, _template_errmsg);
}

/* ============================ template_strerror () ======================== */
char *template_strerror(void)
{
    return _template_errmsg;
}
